import re
import sys

import requests
from bs4 import BeautifulSoup

from pystation.gui import station_gui as gui
from pystation.search.highest_price import all_prices
from pystation.search.search_funcs import search_deal, search_game


# Single game
def single(title, country="en-us", use_gui=None):
    url = f"https://store.playstation.com/{country}/search/{title}"
    # Get the page via requests
    page = requests.get(search_game(url))
    # Create the soup object
    soup = BeautifulSoup(page.content, "html.parser")

    # Search the lowest and highest price
    try:
        disc, price = all_prices(
            soup.find(
                class_="psw-c-bg-card-1 psw-p-y-7 psw-p-x-8 psw-m-sub-x-8 psw-m-sub-x-6@below-tablet-s psw-p-x-6@below-tablet-s"
            )
        )
    except AttributeError:
        sys.exit("The page seems to be invalid. Try specifying the search query.")

    # Add the items to a list in order to use list comprehension later
    items = [soup.find("h1").string, price, disc]

    # If gui is provided
    if use_gui == True:
        # Get image
        image = soup.find(
            "img", class_="psw-blur psw-right-top-third psw-l-fit-cover"
        ).get("src")

        # Display the image
        return gui.main_app(image, price, items[0])

    # Populate the list using list comprehension. Reverse because title is appended after.
    return [[item for item in items if item != None]]


# Sale
def multi(country="en-us", use_gui=None):
    search_url = f"https://store.playstation.com/{country}/pages/deals"
    # Get the page via requests
    page = requests.get(search_deal(search_url))
    # Create the soup object
    soup = BeautifulSoup(page.content, "html.parser")

    try:
        # Get the original price of the game
        discprice = soup.find_all("span", class_="psw-m-r-3")
        # Get the discounted price of the game
        ogprice = soup.find_all(lambda tag: tag.get("class") == ["psw-c-t-2"])
        # Get the title of the game
        title = soup.find_all(
            "span", class_="psw-t-body psw-c-t-1 psw-t-truncate-2 psw-m-b-2"
        )
    except AttributeError:
        sys.exit("The page seems to be invalid.")

    table = []

    if use_gui == True:
        image = soup.find_all("img", class_="psw-blur psw-top-left psw-l-fit-cover")
        for i in range(len(title)):
            url = image[i]["src"].replace(
                str(re.search(r"w=[0-9]+\&thumb=true", image[i]["src"]).group()),
                "w=1920&thumb=false",
            )
            table.append([title[i].text, ogprice[i].text, discprice[i].text, url])
        return table

    else:
        # Append the information to the list
        for i in range(len(title)):
            table.append([title[i].text, ogprice[i].text, discprice[i].text])
        return table



COUNTRY_CODES = {
    "Argentina": {"address": "es-ar"},
    "Australia": {"address": "en-au"},
    "Austria (Österreich)": {"address": "de-at"},
    "Bahrain (English)": {"address": "en-bh"},
    "Belgium (Français)": {"address": "fr-be"},
    "Belgium (Nederlands)": {"address": "nl-be"},
    "Bolivia": {"address": "es-bo"},
    "Brasil": {"address": "pt-br"},
    "Bulgaria (English)": {"address": "en-bg"},
    "Canada": {"address": "en-ca"},
    "Canada (French)": {"address": "fr-ca"},
    "Chile": {"address": "es-cl"},
    "Colombia": {"address": "es-co"},
    "Costa Rica": {"address": "es-cr"},
    "Croatia (English)": {"address": "en-hr"},
    "Cyprus": {"address": "en-cy"},
    "Czech Republic (English)": {"address": "en-cz"},
    "Denmark (Danmark)": {"address": "da-dk"},
    "Ecuador": {"address": "es-ec"},
    "El Salvador": {"address": "es-sv"},
    "Finland (English)": {"address": "en-fi"},
    "France": {"address": "fr-fr"},
    "Germany (Deutschland)": {"address": "de-de"},
    "Greece (English)": {"address": "en-gr"},
    "Guatemala": {"address": "es-gt"},
    "Honduras": {"address": "es-hn"},
    "Hong Kong (English)": {"address": "en-hk"},
    "Iceland (English)": {"address": "en-is"},
    "India": {"address": "en-in"},
    "Indonesia (English)": {"address": "en-id"},
    "Ireland": {"address": "en-ie"},
    "Israel (English)": {"address": "en-il"},
    "Italy": {"address": "it-it"},
    "Japan (日本)": {"address": "ja-jp"},
    "Korea (한국어)": {"address": "ko-kr"},
    "Kuwait (English)": {"address": "en-kw"},
    "Lebanon (English)": {"address": "en-lb"},
    "Luxembourg (Deutsch)": {"address": "de-lu"},
    "Malaysia (English)": {"address": "en-my"},
    "Malta": {"address": "en-mt"},
    "Mexico (México)": {"address": "es-mx"},
    "Nederland": {"address": "nl-nl"},
    "New Zealand": {"address": "en-nz"},
    "Nicaragua": {"address": "es-ni"},
    "Norway (English)": {"address": "en-no"},
    "Oman (English)": {"address": "en-om"},
    "Panama (Panamá)": {"address": "es-pa"},
    "Paraguay": {"address": "es-py"},
    "Peru (Perú)": {"address": "es-pe"},
    "Philippines (English)": {"address": "en-ph"},
    "Poland (English)": {"address": "en-pl"},
    "Portugal": {"address": "pt-pt"},
    "Qatar (English)": {"address": "en-qa"},
    "Romania (English)": {"address": "en-ro"},
    "Saudi Arabia (English)": {"address": "en-sa"},
    "Serbia (Srbija)": {"address": "sr-rs"},
    "Singapore (English)": {"address": "en-sg"},
    "Slovenia (English)": {"address": "en-si"},
    "Slovakia (English)": {"address": "en-sk"},
    "South Africa": {"address": "en-za"},
    "Spain (España)": {"address": "es-es"},
    "Sweden (English)": {"address": "en-se"},
    "Switzerland (Deutsch)": {"address": "de-ch"},
    "Taiwan (English)": {"address": "en-tw"},
    "Thailand (English)": {"address": "en-th"},
    "Ukraine (Українська мова)": {"address": "uk-ua"},
    "United Arab Emirates/ Middle East (English)": {"address": "en-ae"},
    "United States": {"address": "en-us"},
    "United Kingdom: {Great Britain": {"address": "en-gb"},
    "Uruguay": {"address": "es-uy"},
    "Vietnam (English)": {"address": "en-vn"},
}


# Check the country code and return us if not found
def store(country):
    for key in COUNTRY_CODES:
        if country.strip().lower() in key.lower():
            return COUNTRY_CODES[key]["address"]
        else:
            continue

    return "en-us"



import re
import sys

import pytest
from bs4 import BeautifulSoup

from ..cli.cli import main
from ..funcs.currency import store
from ..search.highest_price import all_prices
from ..search.search_funcs import search_store
from ..station import multi, single
from ..gui import Display

"""Regex is used here because the country and language are not necessarily the same for all users.
The title and the price especially may also differ so anything hardcoded won't do here."""


# This is to test the whole working of the cli
@pytest.mark.parametrize("option, game", [("-s", "ezio"), ("-s", "witcher")])
def test_whole(capsys, monkeypatch, option, game):
    with monkeypatch.context() as m:
        m.setattr(sys, "argv", ["pystation/__main__.py", option, game])
        main()
        captured = capsys.readouterr().out
        assert "Original price" in captured and game in captured.lower()


def test_single():
    title, price = single("minecraft")[0]
    assert re.search(r"\w*Minecraft\w*", title)
    assert re.search(r"\D+\d+[,|.]\d+", price)


def test_multi():
    assert len(multi()) > 1
    assert type(multi()[0][0]) == str


def test_invalid_game():
    with pytest.raises(SystemExit):
        single("halo")


def test_price():
    assert all_prices(
        BeautifulSoup(
            '<span data-qa="mfeCtaMain#offer0#finalPrice" class="psw-t-title-m psw-m-r-4">€9,99</span>',
            "html.parser",
        )
    ) == (None, "€9,99")
    assert all_prices(
        BeautifulSoup(
            '<span data-qa="mfeCtaMain#offer0#finalPrice" class="psw-t-title-m psw-m-r-4">€9,99</span><span data-qa="mfeCtaMain#offer1#finalPrice" class="psw-t-title-m psw-m-r-4">Free</span>',
            "html.parser",
        )
    ) == ("Free", "€9,99")


def test_store():
    assert store("Finland") == "en-fi"
    assert store("Brasil") == "pt-br"
    assert store("Great Britain") == "en-gb"



import re


def all_prices(soup):
    prices = soup.find_all(string=re.compile(r"\D*[0-9]+[\.|,][0-9]+"))

    offer_price = offer(soup)

    unique = []
    for price in prices:
        if price.text not in unique and price.text != "":
            unique.append(price.text)
        else:
            continue

    if len(unique) > 1:
        unique = sorted(
            unique, key=lambda price: re.search(r"\w+[\.|,]\w+", price).group(0)
        )
        return unique[0], unique[-1]

    elif offer_price != None and offer_price not in unique:
        if len(unique) > 0:
            return offer_price, unique[0]
        else:
            return offer_price, None

    else:
        return None, unique[0]


def offer(soup):
    try:
        offer_price = soup.find(
            attrs={"data-qa": "mfeCtaMain#offer1#finalPrice"},
        ).string
    except AttributeError:
        try:
            offer_price = soup.find(
                attrs={"data-qa": "mfeCtaMain#offer0#finalPrice"},
            ).string
        except AttributeError:
            return None

    try:
        condition = soup.find(
            "span",
            string=re.compile(r"[+]*PlayStation[+]*"),
            attrs={"data-qa": re.compile(r"mfeCtaMain#offer[0-9]+#discountInfo")},
        ).string
    except AttributeError:
        condition = ""

    if tier := re.search(r"(Premium|Extra)", condition):
        return f"{offer_price} with PlayStation Plus {tier.group(0)}"

    elif "PlayStation Plus" in condition:
        return f"{offer_price} with PlayStation Plus"

    else:
        return offer_price



import argparse
import sys

from tabulate import tabulate

from ..funcs.currency import store
from ..gui.station_gui import main_app
from ..station import multi, single


def main():
    parser = argparse.ArgumentParser(description="Playstation Store price crawler")

    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument(
        "-s",
        "--single",
        help="Get a single game price and information.",
        type=str,
        nargs="+",
    )
    group.add_argument(
        "-gs",
        "--gui_single",
        help="Same as single but with gui and picture",
        type=str,
        nargs="+",
    )
    group.add_argument(
        "-m", "--multi", help="Get all the sale prices and titles", action="store_true"
    )
    group.add_argument(
        "-gm",
        "--gui_multi",
        help="Same as multi but with gui and picture",
        action="store_true",
    )

    parser.add_argument(
        "-c,",
        "--country",
        help="Set country to specify currency.",
        default="United States",
    )

    args = parser.parse_args()

    store_code = store(args.country)

    if args.single:
        table = single(args.single, store_code)

    elif args.gui_single:
        table = single(args.gui_single, store_code, True)
        sys.exit(0)

    elif args.gui_multi:
        table = main_app(multi(store_code, True))
        sys.exit(0)

    elif args.multi:
        table = multi(store_code)

    else:
        sys.exit("Something went wrong. Refer to the help.")
    # Print using tabulate
    print(tabulate(table, headers=["Title", "Original price", "Discounted price"]))


if __name__ == "__main__":
    main()



import inspect
import io
import re
import sys
from tkinter import *

import requests
from PIL import Image, ImageTk


# Make a class in order to scale
class Game:
    def __init__(self, image, price, full_title):
        img = Image.open(io.BytesIO(image))
        # Get the measurements
        width, height = img.size
        # Resize the image to not take the whole screen
        img = img.resize((int(width / 1.3), int(height / 1.3)))

        # Assign the image, price and the title
        self.image = ImageTk.PhotoImage(img)
        self.price = price
        self.full_title = full_title

    def get(self):
        return vars(self)


# Make a class for the navigations
class Navs:
    def __init__(self, max_iter):
        # Iterator to keep track over the current image
        self._iterator = 0
        # to define how far can the the images be iterated
        self.max_iter = max_iter

    @property
    def iterator(self):
        return self._iterator

    @iterator.setter
    def iterator(self, oper):
        # Check whether the iterator would go beyond the boundaries
        if 0 <= self._iterator + oper <= self.max_iter:
            self._iterator = self._iterator + oper
        else:
            pass

    def next_img(self, event, games):
        self.iterator = 1
        game = games[self.iterator]
        img = game.image
        event.config(image=img, text=f"{game.full_title}: {game.price}")
        event.image = img

    def previous_img(self, event, games):
        self.iterator = -1
        game = games[self.iterator]
        img = game.image
        event.config(image=img, text=f"{game.full_title}: {game.price}")
        event.image = img


# Start the gui and check where the function call came from
def main_app(*args):
    root = Tk()
    if inspect.stack()[1][3] == "single":
        Display(root, *args)
        root.mainloop()

    elif inspect.stack()[1][3] == "main":
        Multi_display(root, *args)
        root.mainloop()
    else:
        sys.exit("Invalid function call.")


# Single game display
class Display:
    def __init__(self, master, url, price, full_title):
        self.master = master

        self.game = Game(self.image(url), price, full_title)

        master.title("Game")

        imagelab = Label(
            master,
            image=self.game.image,
            text=f"{full_title}: {price}",
            compound=BOTTOM,
        )
        imagelab.img = self.game.image
        imagelab.config(font=("Times", 25, "bold italic"))
        imagelab.grid(row=0, column=0)

    def image(self, url):
        # Change the image width to 1920 and thumbnail false in the url
        new_url = url.replace(
            str(re.search(r"w=[0-9]+\&thumb=true", url).group()),
            "w=1920&thumb=false",
        )
        # Get the image
        return requests.get(new_url).content


# Discountd games display
class Multi_display:
    def __init__(self, master, table):

        games = list()
        for game in table[0:]:
            img = requests.get(game[3]).content
            games.append(Game(img, game[1], game[0]))

        # Assign a title
        master.title("Discounted Games")

        imagelab = Label(
            master,
            image=games[0].image,
            text=f"{games[0].full_title}: {games[0].price}",
            compound=BOTTOM,
        )
        imagelab.config(font=("Times", 25, "bold italic"))
        imagelab.pack(fill=BOTH, expand=True)

        # Create a Navs object
        navs = Navs(len(games) - 1)

        btnfr = Button(
            master, text="Next Game", command=lambda: navs.next_img(imagelab, games)
        )
        btnfr.pack(side=RIGHT)

        btnpr = Button(
            master,
            text="Previous Game",
            command=lambda: navs.previous_img(imagelab, games),
        )
        btnpr.pack(side=LEFT)



from pystation.gui import station_gui


"""These are tests to be run manually for the testing of the tkinter features.
These are just some things to run and can be easily modified to the users liking"""
table = [
    [
        "LEGO® Star Wars™:The Skywalker Saga Deluxe Edition PS4 & PS5",
        "€52,46",
        "€69,95",
    ],
    ["WWE 2K22 for PS4™", "€38,47", "€69,95"],
    ["WWE 2K22 for PS5™", "€44,97", "€74,95"],
    ["Call of Duty®: Vanguard - Cross-Gen Bundle", "€39,97", "€79,95"],
    ["God of War™", "€9,97", "€19,95"],
    ["STAR WARS Jedi: Fallen Order™", "€9,99", "€49,95"],
    ["Grand Theft Auto V: Premium Edition", "€14,67", "€34,95"],
    ["The Quarry for PS4™", "€46,86", "€69,95"],
    ["The Quarry for PS5™", "€50,21", "€74,95"],
    ["Red Dead Redemption 2", "€23,98", "€59,95"],
    ["It Takes Two PS4™ & PS5™", "€19,97", "€39,95"],
]
t = [
    "LEGO® Star Wars™:The Skywalker Saga Deluxe Edition PS4 & PS5",
    "€52,46",
    "€69,95",
    "https://image.api.playstation.com/vulcan/ap/rnd/202207/1210/6zhGBKQpPrlLNI2a7EfALNs1.png?w=1920&thumb=true",
], [
    "WWE 2K22 for PS4™",
    "€38,47",
    "€69,95",
    "https://image.api.playstation.com/vulcan/ap/rnd/202205/2800/1inUo7VTdbmAD9c1efdZGB9R.png?w=1920&thumb=false",
]
print(t[0][3], t[0][0], t[0][1])
station_gui.display(t[0][3], t[0][0], t[0][1])
station_gui.multi_display(t)



import re
import sys

import requests
from bs4 import BeautifulSoup


# Get the latest sale
def search_store(url):
    # Pass a user agent so the server will serve the proper site
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/104.0.0.0 Safari/537.36"
    }
    # Create soup object
    soup = BeautifulSoup(requests.get(url, headers=headers).content, "html.parser")

    # Assure that the search wasn't empty
    if soup.find("div", attrs={"data-qa": "search-no-results"}):
        sys.exit("No search results found. Try specifying the search query.")

    #
    return soup
    # Return f-string containg the url for the sale


# Check that the search result is a game
def search_game(url):
    soup = search_store(url)
    game_tags = soup.find_all("a", class_="psw-link psw-content-link")
    for i in range(len(game_tags)):
        if not (
            tag := game_tags[i].find(
                "span",
                class_="psw-product-tile__product-type psw-t-bold psw-t-size-1 psw-t-truncate-1 psw-c-t-2 psw-t-uppercase psw-m-b-1",
                attrs={
                    f"'data-qa':'{re.compile(r'search#productTile[0-9]{2}#product-type')}'"
                },
            )
        ) or tag.string in ["Pre-Order", "Game Bundle"]:
            return f"https://store.playstation.com{game_tags[i].get('href')}"
        else:
            continue

    sys.exit("Page seems to be invalid. Try specifying the search query.")


def search_deal(url):
    soup = search_store(url)
    href = soup.find(
        "a",
        class_="psw-link psw-quick-action-link psw-button psw-b-0 psw-t-button psw-l-line-center psw-button-sizing psw-button-sizing--medium psw-button-sizing--icon-only psw-quick-action-button",
    ).get("href")
    return f"https://store.playstation.com{href}"
